package com.king.apps.pepperpanic.sound.ambient
{
    import com.king.apps.pepperpanic.shared.util.RandomUtil;
    import com.king.apps.pepperpanic.sound.GameSoundChannelEnum;
    import com.king.apps.pepperpanic.sound.GameSoundService;
    import com.king.apps.pepperpanic.sound.library.SoundsLibrary;
    import com.king.apps.pepperpanic.sound.player.ManagedSoundChannel;

    import flash.events.Event;
    import flash.utils.Dictionary;
    import flash.utils.setTimeout;

    /**
     * Uses the SoundEffectsPlayerService to start a background loop and then play a selection of background noises according
     * to provided parameters.
     */
    public class AmbientSoundSystem
    {
        public static const DEFAULT_AMBIENT_SET : String = "1";

        private var _soundLibrary : SoundsLibrary;
        private var _soundEffectPlayerService : GameSoundService;
        private var _channelToBackgroundElement : Dictionary;
        private var _enabled : Boolean;

        public function AmbientSoundSystem(soundEffectsPlayerService : GameSoundService)
        {
            _soundLibrary = soundEffectsPlayerService.getSoundLibrary();
            _soundEffectPlayerService = soundEffectsPlayerService;
            _channelToBackgroundElement = new Dictionary();
        }

        public function playAmbientSoundSet(soundSetId : String) : void
        {
            _enabled = true;

            if (_soundLibrary && _soundLibrary.hasAmbientSetEntry(soundSetId))
            {
                var ambientSoundSet : AmbientSoundSetEntry = _soundLibrary.acquireAmbientSetEntry(soundSetId);

                playBackgroundLoop(ambientSoundSet.getAmbientLoopSound());
                startBackgroundElementsService(ambientSoundSet);
            }
        }

        public function stopAllAmbientsound() : void
        {
            for (var key : Object in _channelToBackgroundElement)
            {
                var channel : ManagedSoundChannel = ManagedSoundChannel(key);
                channel.stop();
                channel.removeEventListener(Event.SOUND_COMPLETE, soundChannelComplete);
            }

            _channelToBackgroundElement = new Dictionary();
            _enabled = false;
        }

        private function playBackgroundLoop(ambientLoopSound : AmbientLoopSound) : void
        {
            _soundEffectPlayerService.playLoopedSoundUsingConfigLibrary(GameSoundChannelEnum.AMBIENT,ambientLoopSound.getId(),
                                                                        ambientLoopSound.getVolume(),
                                                                        ambientLoopSound.getPan());
        }

        private function startBackgroundElementsService(ambientSoundSet : AmbientSoundSetEntry) : void
        {
            for each(var backgroundElement : BackgroundElement in ambientSoundSet.getBackgroundElements())
            {
                playBackgroundElement(backgroundElement);
            }
        }

        private function playBackgroundElement(backgroundElement : BackgroundElement) : void
        {
            const id : String = createIdFromVariationsRandomly(backgroundElement);
            const startDelayInSeconds : Number = RandomUtil.randomBetweenNumber(backgroundElement.getDelayRangeMin(), backgroundElement.getDelayRangeMax());
            const volume : Number = RandomUtil.randomBetweenNumber(backgroundElement.getVolumeMin(), backgroundElement.getVolumeMax());
            const pan : Number = RandomUtil.randomBetweenNumber(backgroundElement.getPanMin(), backgroundElement.getPanMax());

            setTimeout(playAndTrackSound, startDelayInSeconds * 1000,backgroundElement,id, volume, pan);

        }

        private function playAndTrackSound(backgroundElement : BackgroundElement, id : String, volume : Number, pan : Number) : void
        {
            //Guard against rogue timeouts without having to keep track of a bunch of handle mappings.
            if (!_enabled)
            {
                return;
            }

            var managedSoundChannel : ManagedSoundChannel = _soundEffectPlayerService.playSoundUsingConfigLibrary(GameSoundChannelEnum.AMBIENT,id, volume, pan);

            _channelToBackgroundElement[managedSoundChannel] = backgroundElement;
            managedSoundChannel.addEventListener(Event.SOUND_COMPLETE, soundChannelComplete, false, 0, true);

        }

        private function soundChannelComplete(event : Event) : void
        {
            var key : ManagedSoundChannel = ManagedSoundChannel(event.target);
            key.removeEventListener(Event.SOUND_COMPLETE, soundChannelComplete);

            var element : BackgroundElement = _channelToBackgroundElement[key];
            playBackgroundElement(element);
        }

        private function createIdFromVariationsRandomly(backgroundElement : BackgroundElement) : String
        {

            const variations : uint = backgroundElement.getVariations();
            const baseId : String = backgroundElement.getBaseId();
            const x : int = RandomUtil.randomBetweenInt(1, variations + 1);
            const id : String = (x < 10) ? (baseId + "_0" + x) : baseId + "_" + x;

            return id;
        }

        public function ambientSoundSetExistsForId(switcherBackgroundSoundSetId : String) : Boolean
        {
            if (_soundLibrary)
            {
                return _soundLibrary.hasAmbientSetEntry(switcherBackgroundSoundSetId);
            }
            return false;
        }

    }
}
